<?php

namespace Drupal\navu\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Render\Markup;

/**
 * Provides a 'Navu Sidebar' Block.
 *
 * @Block(
 *   id = "navu_block",
 *   admin_label = @Translation("NAVU Sidebar"),
 *   category = @Translation("AI"),
 * )
 */
class NavuSidebarBlock extends BlockBase implements BlockPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $site_code = $config['navu_site_code'] ?? '';
    // Styling
    $styling['--nv-font-family'] = $config['navu_font_family'] ?? '';
    $styling['--nv-primary'] = $config['navu_primary'] ?? '';
    $styling['--nv-secondary'] = $config['navu_secondary'] ?? '';
    $styling['--nv-highlight-surface-high'] = $config['navu_highlight_surface_high'] ?? '';
    $styling['--nv-on-highlight-surface-high'] = $config['navu_on_highlight_surface_high'] ?? '';
    $styling['--nv-on-surface'] = $config['navu_on_surface'] ?? '';
    $styling['--nv-sidebar-suggestion-bg'] = $config['navu_sidebar_suggestion_bg'] ?? '';
    $styling['--nv-sidebar-suggestion-fg'] = $config['navu_sidebar_suggestion_fg'] ?? '';

    if (empty($site_code)) {
      return [
        '#markup' => $this->t('Please configure the Navu site code in the block settings.'),
      ];
    }

    $scripts = $this->getNavuScript($site_code, $styling);

    return [
      '#attached' => [
        'html_head' => [
          [
            [
              '#type' => 'html_tag',
              '#tag' => 'script',
              '#value' => Markup::create($scripts['js']),
              '#attributes' => [
                'type' => 'text/javascript',
              ],
            ],
            'navu_widget_script',
          ],
          [
            [
              '#type' => 'html_tag',
              '#tag' => 'style',
              '#value' => Markup::create($scripts['css']),
            ],
            'navu_widget_custom_styling',
          ],
        ],
      ],
      '#cache' => [
        'contexts' => ['url'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    $form['navu_site_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Site Code'),
      '#description' => $this->t('Enter your NAVU site code (7 digit alphanumeric). <a href="@navudocs" _target="blank">See NAVU documentation for more information.</a>', ['@navudocs' => 'https://navu.co/where-to-find-my-site-code/']),
      '#default_value' => $config['navu_site_code'] ?? '',
      '#required' => TRUE,
    ];

    $form['navu_font_family'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Font Family'),
      '#description' => $this->t('Enter the font family to use in NAVU sidebar. Must be loaded in your site somehow (ie via CSS).'),
      '#default_value' => $config['navu_font_family'] ?? '',
    ];

    $form['navu_primary'] = [
      '#type' => 'color',
      '#title' => $this->t('Primary Color'),
      '#description' => $this->t('Color of buttons.'),
      '#default_value' => $config['navu_primary'] ?? '',
    ];

    $form['navu_secondary'] = [
      '#type' => 'color',
      '#title' => $this->t('Secondary Color'),
      '#description' => $this->t('Color of citations on hover, message pulsing color.'),
      '#default_value' => $config['navu_secondary'] ?? '',
    ];

    $form['navu_highlight_surface_high'] = [
      '#type' => 'color',
      '#title' => $this->t('Highlight Surface High Color'),
      '#description' => $this->t('Background of suggestion question, tabs.'),
      '#default_value' => $config['navu_highlight_surface_high'] ?? '',
    ];

    $form['navu_on_highlight_surface_high'] = [
      '#type' => 'color',
      '#title' => $this->t('On Highlight Surface High Color'),
      '#description' => $this->t('Foreground of suggestion question, tabs.'),
      '#default_value' => $config['navu_on_highlight_surface_high'] ?? '',
    ];

    $form['navu_on_surface'] = [
      '#type' => 'color',
      '#title' => $this->t('On surface color'),
      '#description' => $this->t(''),
      '#default_value' => $config['navu_on_surface'] ?? '',
    ];

    $form['navu_sidebar_suggestion_bg'] = [
      '#type' => 'color',
      '#title' => $this->t('Sidebar suggestion background color'),
      '#description' => $this->t(''),
      '#default_value' => $config['navu_sidebar_suggestion_bg'] ?? '',
    ];

    $form['navu_sidebar_suggestion_fg'] = [
      '#type' => 'color',
      '#title' => $this->t('Sidebar suggestion foreground (text) color'),
      '#description' => $this->t(''),
      '#default_value' => $config['navu_sidebar_suggestion_fg'] ?? '',
    ];

    $form['navu_styling_help'] = [
      '#type' => 'markup',
      '#title' => $this->t('Navu Styling'),
      '#markup' => $this->t('Visit <a target="_blank" href="@navudocs">the official styling documentation</a> for details on colors.', ['@navudocs' => 'https://hivepoint.github.io/navu-docs/sidebar-styling.html']),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('label_display', $form_state->getValue('block_label_display'));
    $this->setConfigurationValue('navu_site_code', $form_state->getValue('navu_site_code'));;
    $this->setConfigurationValue('navu_font_family', $form_state->getValue('navu_font_family'));;
    $this->setConfigurationValue('navu_primary', $form_state->getValue('navu_primary'));;
    $this->setConfigurationValue('navu_secondary', $form_state->getValue('navu_secondary'));;
    $this->setConfigurationValue('navu_highlight_surface_high', $form_state->getValue('navu_highlight_surface_high'));;
    $this->setConfigurationValue('navu_on_highlight_surface_high', $form_state->getValue('navu_on_highlight_surface_high'));;
    $this->setConfigurationValue('navu_on_surface', $form_state->getValue('navu_on_surface'));;
    $this->setConfigurationValue('navu_sidebar_suggestion_bg', $form_state->getValue('navu_sidebar_suggestion_bg'));;
    $this->setConfigurationValue('navu_sidebar_suggestion_fg', $form_state->getValue('navu_sidebar_suggestion_fg'));;
  }

  /**
   * Generate the Navu script with the site code.
   * https://navu.co/installing-navu/
   *
   * @param string $site_code
   *   The site code to inject into the script.
   *
   * @return array
   *   The complete Navu script and styling customizations.
   */
  private function getNavuScript($site_code, $styling) {
    // Properly escape the site code for JavaScript
    $escaped_site_code = Html::escape($site_code);

    $scripts['js'] = '"use strict";(o=>{const e=document,a=(t,...n)=>{const s=e.head.appendChild(e.createElement(t));return n.forEach(i=>s.setAttribute(i[0],i[1])),s};a("meta",["name","navu:site"],["content",o]),a("script",["src","https://embed.navu.co/boot.js"],["async","true"]);const d=sessionStorage.getItem("nv-sidebar");if(d){const t=JSON.parse(d);t.css&&(a("style",["data-widget","sidebar"]).innerHTML=t.css);const n=i=>e.body.classList.add("nv-sidebar-"+i),s=e.readyState;s==="interactive"||s==="complete"?n(t.state):e.addEventListener("DOMContentLoaded",()=>n(t.state))}})("' . $escaped_site_code . '");';

    $scripts['css'] = 'navu-sidebar {';
    foreach ($styling as $key => $value) {
      if (!empty($value)) {
        $scripts['css'] .= $key . ': ' . $value . ';';
      }
    }
    $scripts['css'] .= '}';

    return $scripts;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    // Cache for 1 hour since we expect the script content is static.
    return 3600;
  }

}
